﻿using System;
using AspNetCoreSsrTemplateEngine.DataBoundContext;
using AspNetCoreSsrTemplateEngine.Renders;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;

namespace AspNetCoreSsrTemplateEngine.Template
{
#if !EMBED_HTML

    public class HotLoadTemplate : ITemplate,IDisposable
    {
        private readonly string _fullTypeName;
        private readonly IComponentResolver _componentResolver;
        private readonly FileSystemWatcher _fileSystemWatcher;

        public IReadOnlyList<IRender> RenderSequence { get; private set; }

        public HotLoadTemplate(string fullTypeName, IComponentResolver componentResolver)
        {
            _fullTypeName = fullTypeName;
            _componentResolver = componentResolver;
            Reload();
            var templateFilePath = TemplateLoader.GetTemplateFilePath(fullTypeName);
            _fileSystemWatcher = new FileSystemWatcher(templateFilePath);
            _fileSystemWatcher.Changed += _fileSystemWatcher_Changed;
        }

        private void _fileSystemWatcher_Changed(object sender, FileSystemEventArgs e)
        {
            if (e.ChangeType == WatcherChangeTypes.Changed) Reload();
        }

        private void Reload()
        {
            RenderSequence = TemplateLoader.LoadByFullTypeName(_fullTypeName, _componentResolver);
        }

        public async Task RenderAsync(IReadonlyDataBindContext dataBindContext, Stream stream)
        {
            foreach (var render in RenderSequence)
            {
                await render.RenderAsync(dataBindContext, stream);
            }
        }

        public void Dispose()
        {
            _fileSystemWatcher?.Dispose();
        }
    }

#endif
}
